xquery version "3.0";

module namespace bc = "http://boredcat";

import module namespace rest = "http://exquery.org/ns/restxq";
import module namespace templates = "http://exist-db.org/xquery/templates" at "lib/templates.xql";
import module namespace util = "http://exist-db.org/xquery/util";
import module namespace xmldb = "http://exist-db.org/xquery/xmldb";

declare namespace http = "http://expath.org/ns/http-client";
declare namespace json = "http://www.json.org";
declare namespace output = "http://www.w3.org/2010/xslt-xquery-serialization";

declare variable $bc:db-collection-base := "/db/publishing";
declare variable $bc:db-collection-articles := $bc:db-collection-base || "/articles";
declare variable $bc:doc-article-status := $bc:db-collection-articles || "/status.xml";
declare variable $bc:db-collection-editorial := $bc:db-collection-articles || "/editorial";
declare variable $bc:db-collection-editorial-inbox := $bc:db-collection-editorial || "/inbox";
declare variable $bc:db-collection-editorial-process := $bc:db-collection-editorial || "/process";
declare variable $bc:db-collection-assembly := $bc:db-collection-articles || "/assembly";
declare variable $bc:db-collection-assembly-inbox := $bc:db-collection-assembly|| "/inbox";
declare variable $bc:db-collection-assembly-process := $bc:db-collection-assembly|| "/process";

declare variable $bc:templates-config := map {
    $templates:CONFIG_PARAM_RESOLVER := bc:templates-param-resolver#1,
    $templates:CONFIG_APP_ROOT := $bc:db-collection-base
};



declare
    %rest:GET
    %rest:path("/boredcat")
    %output:method("html5")
    %output:media-type("text/html")
function bc:home() {

    bc:template(
        doc("/db/publishing/home.html"),
        map {
            "page" := "home"
        }
    )
};


declare
    %rest:POST
    %rest:path("/boredcat/login")
    %rest:form-param("username", "{$username}")
    %rest:form-param("password", "{$password}")
    %output:method("html5")
    %output:media-type("text/html")
function bc:login($username as xs:string*, $password as xs:string*) {
    let $authenticated := xmldb:is-authenticated() or xmldb:login($bc:db-collection-base, $username, $password)
    return
        if($authenticated)then
            document {
                element rest:response {
                
                    let $dest := if(sm:get-user-groups($username)[ . = "editors"])then
                        "/editor"
                    else
                        "/author"
                    return
                        <http:response status="303">
                            <http:header name="Location" value="{rest:base-uri() || '/boredcat' || $dest}"/>
                        </http:response>
                }
            }
        else
            bc:http-cat-response(401, ())
};


declare
    %rest:GET
    %rest:path("/boredcat/author")
    %output:method("html5")
    %output:media-type("text/html")
function bc:author() {
    bc:template(
        doc("/db/publishing/author.html"),
        map {
            "page" := "author"
        }
    )
};


declare
    %rest:POST("{$body}")
    %rest:path("/boredcat/author/submission")
    %rest:form-param("article", "{$article}")
function bc:author-submission($article, $body) {
    <ok>{$body}</ok>
    (: TODO upload and unpack zip file :)
};


declare
    %rest:GET
    %rest:path("/boredcat/author/articles")
    %output:method("json")
    %output:media-type("application/json")
function bc:author-articles() {
    bc:article-status(function($context){
        $context/@author = xmldb:get-current-user()
    })
};

declare
    %rest:GET
    %rest:path("/boredcat/editor")
    %output:method("html5")
    %output:media-type("text/html")
function bc:editor() {
    bc:template(
        doc("/db/publishing/editor.html"),
        map {
            "page" := "editor"
        }
    )
};


declare
    %rest:GET
    %rest:path("/boredcat/editor/articles")
    %output:method("json")
    %output:media-type("application/json")
function bc:editor-articles($article, $body) {
    bc:article-status(function($context){ true() })
};

declare
    %rest:POST
    %rest:path("/boredcat/article/{$uuid}/start")
function bc:start-editorial($uuid) {
    
    (: move from inbox to editorial collection :)
    let $moved := xmldb:move($bc:db-collection-editorial-inbox || "/" || $uuid, $bc:db-collection-editorial-process)
    return
    
        (: record that we have moved the document to editorial process :)
        let $updated := update insert
                <editorial-process timestamp="{current-dateTime()}">{$bc:db-collection-editorial-process || "/" || $uuid}</editorial-process>
            into doc(xs:anyURI($bc:doc-article-status))/articles/article[@id eq $uuid]
        return
        
            <rest:response>
                <http:response status="303">
                    <http:header name="Location" value="{rest:base-uri() || '/boredcat/editor'}"/>
                </http:response>
            </rest:response>
};


declare
    %rest:GET
    %rest:path("/boredcat/article/{$uuid}/edit")
    %output:method("html5")
    %output:media-type("text/html")
function bc:editorial-edit($uuid) {
    bc:template(
        doc("/db/publishing/edit.html"),
        map {
            "page" := "edit",
            "uuid" := $uuid
        }
    )
};


declare
    %rest:HEAD
    %rest:path("/boredcat/article/{$uuid}/edit")
    %output:method("html5")
    %output:media-type("text/html")
function bc:editorial-edit-head($uuid) {
    let $source-collection := $bc:db-collection-editorial-process || "/" || $uuid
    let $source-resource := xmldb:get-child-resources($source-collection)[ends-with(., ".xml")][1]       (: TODO currently only opens first XML file, should look for DocBook! :)
    return
    
        <rest:response>
            <http:response status="200">
                <http:header name="Last-Modified" value="{xmldb:last-modified($source-collection, $source-resource)}"/>
            </http:response>
        </rest:response>
};


declare function bc:xopus-edit-pipeline($node as node(), $model as map(*)) {
    let $uuid := $model("uuid")
    let $source-collection := $bc:db-collection-editorial-process || "/" || $uuid
    let $source-resource := xmldb:get-child-resources($source-collection)[ends-with(., ".xml")][1]       (: TODO currently only opens first XML file, should look for DocBook! :)
    let $source := $source-collection || "/" || $source-resource
    return
    
    <div xopus="true">
        Starting Xopus...
        <xml>
            <config version="1.0" xmlns="http://www.xopus.com/xmlns/config">
                <!-- import src="config.xml"/ -->
                <miscellaneous>
                    <debugMode>true</debugMode>
                </miscellaneous>
 
                <pipeline xml="/exist/rest{$source}" xsd="/exist/rest{$bc:db-collection-base}/validation/docbook.xsd">
                    <view name="WYSIWYG View">
                        <transform xsl="/exist/rest{$bc:db-collection-base}/validation/docbook.xslt"/>
                    </view>
                </pipeline>
            </config>
        </xml>
    </div>
};


declare
    %rest:POST
    %rest:path("/boredcat/article/{$uuid}/finish")
function bc:finish-editorial($uuid) {
    
    (: move from inbox to editorial collection :)
    let $moved := xmldb:move($bc:db-collection-editorial-process || "/" || $uuid, $bc:db-collection-assembly-inbox)
    return
    
        (: record that we have moved the document to assembly inbox :)
        let $updated := update insert
                <assembly-inbox timestamp="{current-dateTime()}">{$bc:db-collection-assembly-inbox || "/" || $uuid}</assembly-inbox>
            into doc(xs:anyURI($bc:doc-article-status))/articles/article[@id eq $uuid]
        return
        
            <rest:response>
                <http:response status="303">
                    <http:header name="Location" value="{rest:base-uri() || '/boredcat/editor'}"/>
                </http:response>
            </rest:response>
};



declare
    %rest:GET
    %rest:path("/boredcat/article/{$uuid}/pdf")
    %output:method("binary")
    %output:media-type("application/pdf")
function bc:article-pdf-preview($uuid) {
    
    (: move from inbox to editorial collection :)
    let $source-collection := $bc:db-collection-assembly-process || "/" || $uuid
    let $source-resource := xmldb:get-child-resources($source-collection)[ends-with(., ".xml")][1]       (: TODO currently only opens first XML file, should look for DocBook! :)
    let $source := $source-collection || "/" || $source-resource
    return
    
    
        let $fo := transform:transform(doc($source), doc("/db/publishing/validation/docbook-fo.xslt"), ())
        return
	
	       xslfo:render($fo, "application/pdf", ())
};


declare function bc:user-human-name($node as node(), $model as map(*)) as element(span) {
    let $human-name := sm:get-account-metadata(xmldb:get-current-user(), xs:anyURI("http://axschema.org/namePerson")) return
        <span>
        {
            if(exists($human-name))then
                $human-name
            else
                xmldb:get-current-user()
        }
        </span>
};



(: for any unknown resources we redirect the client to the REST Server :)
declare
    %rest:GET
function bc:default() {
    let $relative-uri := replace(rest:uri(), rest:base-uri(), "") return
        if(starts-with($relative-uri, "/lib") or starts-with($relative-uri, "/boredcat/lib"))then 
            let $db-path := concat($bc:db-collection-base, replace($relative-uri, "/boredcat", "")), 
            (: replace($relative-uri, "/boredcat", $db-collection-base), :)
            $rest-server-uri := concat("/exist/rest", $db-path)
            return
            
                (: <p>forwad to rest server {rest:base-uri()} and {rest:uri()} and {$rest-server-uri}</p> :)
            
                document {
                    <rest:response>
                        <http:response status="303">
                            <http:header name="Location" value="{$rest-server-uri}"/>
                        </http:response>
                    </rest:response>
                }
        else
                bc:http-cat-response(404, "Could not find the resource: " || rest:uri())
};

declare
    %private
function bc:http-cat-response($http-code as xs:int, $message as xs:string?) {
    (
        document {
            <rest:response>
                <http:response status="{$http-code}">
                    <http:header name="Content-Type" value="text/html"/>
                </http:response>
            </rest:response>
        }
        ,
        <html>
            <title>{$http-code}</title>
            <body>
                <div style="text-align: center;">
                    <p>
                        <img src="/exist/restxq/boredcat/lib/{$http-code}.jpg"/>
                    </p>
                    <p>{$message}</p>
                </div>
            </body>
        </html>
    )
};

declare 
    %private
function bc:article-status($predicate as function(item()*) as xs:boolean) {
    let $articles := doc($bc:doc-article-status)/articles/article[$predicate(.)]
    return
        <articles total="{count($articles)}">
        {
            for $article in $articles
            return
                <article>
                    <id>{$article/string(@id)}</id>
                    <name>{$article/submission/replace(., ".*/", "")}</name>
                    <status>
                    {
                        if(exists($article/assembly-process))then
                            "In Assembly"
                        else if(exists($article/assembly-inbox))then
                            "Awaiting Assembly"
                        else if(exists($article/editorial-process))then
                            "In Editorial"
                        else if(exists($article/editorial-inbox))then
                            "Awaiting Editorial"
                        else
                            "Failed Validation"
                        
                    }</status>
                    <author>{$article/string(@author)}</author>
                    <timestamp>{$article/submission/string(@timestamp)}</timestamp>
                </article>
        }
        </articles>
};


(: simple wrapper around templates:apply for our purposes :)
declare
    %private
function bc:template($content) {
    bc:template($content, ())
};


(: simple wrapper around templates:apply for our purposes :)
declare
    %private
function bc:template($content, $model) {
  templates:apply(
        $content,
        bc:templates-fn-resolver#2,
        $model,
        $bc:templates-config
    )
};


(: needed to lookup our functions from the templates module :)
declare function bc:templates-fn-resolver($functionName as xs:string, $arity as xs:int) {
    try {
        function-lookup(xs:QName($functionName), $arity)
    } catch * {
        ()
    }
};


declare function bc:templates-param-resolver($arg as xs:string) as item()* {
    ()
};
